"use strict";
// * Fetch the entity configurations, until the trust anchors are hit
// * Fetch the entity statements back until the entityId is hit
// * Merge and apply the policies, trickeling down
// * Return a list of trust chains where the policies are applied, ending up at the `entityId` again
// * Errors
//    * when no trust anchor could be found
//    * when no trust chain with valid applied could be found
// resolveTrustChains(entityId: string, trustAnchorEntityIds: Array<string>) -> Promise<Array<TrustChain>>
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveTrustChains = void 0;
const entityConfiguration_1 = require("../entityConfiguration");
const entityStatement_1 = require("../entityStatement");
const ErrorCode_1 = require("../error/ErrorCode");
const OpenIdFederationError_1 = require("../error/OpenIdFederationError");
// TODO: Apply the policies
// TODO: Look into what we want to return in this function. Because the entity configuration is also very valuable
// TODO: We might also need to return the entity configuration which has all the policies applied. So that a chain has both the statements and the configuration
/**
 * Resolves the trust chains for the given entityId and trust anchor entityIds.
 *
 * @param options - The options for the trust chain resolution.
 * @returns A promise that resolves to an array of trust chains. (The first item in the trust chain is the leaf entity configuration, the last item is the trust anchor entity configuration)
 */
const resolveTrustChains = (options) => __awaiter(void 0, void 0, void 0, function* () {
    const { entityId, trustAnchorEntityIds, verifyJwtCallback } = options;
    const now = new Date();
    const entityConfigurationChains = yield (0, entityConfiguration_1.fetchEntityConfigurationChains)({
        leafEntityId: entityId,
        trustAnchorEntityIds,
        verifyJwtCallback,
    });
    const trustChains = [];
    for (const entityConfigurationChain of entityConfigurationChains) {
        // The last item in the chain is the trust anchor's entity configuration
        const entityStatementChain = yield (0, entityStatement_1.fetchEntityStatementChain)({
            entityConfigurations: entityConfigurationChain,
            verifyJwtCallback,
        });
        if (entityStatementChain.some((statement) => statement.exp < now)) {
            // Skip expired chains
            continue;
        }
        // TODO: Merge all the policies and check them against the metadata of the leaf entity
        const leafEntityConfiguration = entityConfigurationChain[0];
        // Should never happen but for the type safety
        if (!leafEntityConfiguration)
            throw new OpenIdFederationError_1.OpenIdFederationError(ErrorCode_1.ErrorCode.Validation, 'No leaf entity configuration found');
        const trustAnchorEntityConfiguration = entityConfigurationChain[entityConfigurationChain.length - 1];
        if (!trustAnchorEntityConfiguration)
            throw new OpenIdFederationError_1.OpenIdFederationError(ErrorCode_1.ErrorCode.Validation, 'No trust anchor entity configuration found');
        trustChains.push({
            chain: entityStatementChain,
            trustAnchorEntityConfiguration,
            leafEntityConfiguration,
        });
    }
    return trustChains;
});
exports.resolveTrustChains = resolveTrustChains;
//# sourceMappingURL=resolveTrustChains.js.map